// Sudokus con Clase
// Resolucin de sudokus con tcnicas reales
// (C) 2022 Salvador Pozo
// Con Clase https://conclase.net
//
// Declaracin de la clase Tablero

#ifndef SUDOKU_TABLERO
#define SUDOKU_TABLERO

#include <fstream>
#include "casilla.h"
#include "fila.h"
#include "columna.h"
#include "caja.h"
#include "lista.h"
#include "desnudo.h"

class Tablero {
    private:
    Casilla casilla[9][9];
    Fila fila[9];
    Columna columna[9];
    Caja caja[9];
    int cuenta;
    std::ofstream flog;
    Lista<Desnudo> desnudo;

    bool BuscaParesDesnudosFila(int f);
    bool BuscaParesDesnudosColumna(int c);
    bool BuscaParesDesnudosCaja(int c);

    bool ProcesaParesDesnudosFila(int f);
    bool ProcesaParesDesnudosColumna(int c);
    bool ProcesaParesDesnudosCaja(int c);

    public:
    Tablero(char* t);
    ~Tablero() { flog.close(); }
    void Log(char *cad);
    Casilla &GetCasilla(int f, int c) { return casilla[f][c]; }
    const int Cuenta() { return cuenta; }
    void Mostrar();
    bool Soluciones();
    bool Valido(int f, int c, int v);
    void Asignar(int f, int c, int v);

    // Resolucin:
    void LimpiaResaltes();
    void LimpiaCriba();
    void Criba(int v);
    bool ProcesarCriba(int v);
    bool CandidatoUnico();
    bool ProcesarCribaSolitarios(int v);
    bool BuscaParesDesnudos();
    bool ProcesaParesDesnudos();

    Lista<Desnudo> *ListaDesnudo() { return &desnudo; }

};
#endif // SUDOKU_TABLERO
